"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const generator_helper_1 = require("@prisma/generator-helper");
const multilines_1 = __importDefault(require("multilines"));
const path = __importStar(require("path"));
const rimraf_1 = __importDefault(require("rimraf"));
const typescript_1 = require("typescript");
const util_1 = require("util");
const vfs_1 = require("./vfs");
const seed_1 = require("./intellisense/seed");
const pool_1 = require("./intellisense/pool");
const del = util_1.promisify(rimraf_1.default);
/**
 * Generates prisma-test-utils library using Prisma Generators.
 * The function presumes there's an existing photon library at an absolute path.
 *
 * @param options
 */
function generatePrismaTestUtils(options) {
    return __awaiter(this, void 0, void 0, function* () {
        /* Config */
        const client = options.otherGenerators.find(og => og.provider === 'prisma-client-js');
        /* istanbul ignore next */
        if (!client) {
            throw new Error(`You need to generate Prisma Client first.`);
        }
        const photonPath = client.output;
        const outputDir = options.generator.output;
        try {
            yield del(outputDir, {});
        }
        catch (err) /* istanbul ignore next */ {
            console.log(`Error while copying Runtime`);
            console.error(err);
            return '';
        }
        /* Static files. */
        const staticFs = {
            [path.join(outputDir, './static/index.js')]: eval(`path.join(
      __dirname,
      '../runtime/index.js',
    )`),
            [path.join(outputDir, './static/')]: eval(`path.join(
      __dirname,
      '../runtime',
    )`),
        };
        try {
            yield vfs_1.copyVFS(staticFs);
        }
        catch (err) /* istanbul ignore next */ {
            console.log(`Error while copying Runtime`);
            console.error(err);
            return '';
        }
        /**
         * The generation process is separated into three parts:
         *  1. We use TS to build the dynamic parts of the library which refer to the
         *    static ones.
         *  2. We wire the functions from the static part with the dynamic properties.
         *  3. We write the files to the file system.
         */
        /* Dynamic files */
        const dmmf = require(photonPath).dmmf;
        const seedLib = multilines_1.default `
  | import { PrismaClient, dmmf } from '${photonPath}';
  | import { getSeed, SeedModels, SeedFunction } from './static';
  |
  | interface GeneratedSeedModels extends SeedModels {
  |   ${seed_1.generateGeneratedSeedModelsType(dmmf)}
  | }
  |
  | export const seed: SeedFunction<PrismaClient, GeneratedSeedModels>  = getSeed<PrismaClient, GeneratedSeedModels>(dmmf);
  | export default seed
  | export { GeneratedSeedModels }
  | export {
  |   SeedOptions,
  |   SeedModelsDefinition,
  |   SeedKit,
  |   SeedModels,
  |   SeedModel,
  |   ID,
  |   SeedModelFieldDefinition,
  |   SeedModelFieldRelationConstraint,
  | } from './static'
  `;
        const poolLib = multilines_1.default `
  | import { dmmf } from '${photonPath}';
  | import { getMySQLPool, getPostgreSQLPool, getSQLitePool } from './static';
  |
  | export default ${pool_1.generatePoolType(options)}(dmmf, "${__dirname}");
  | export { MySQLPoolOptions, PostgreSQLPoolOptions, SQLitePoolOptions } from './static'
  | export { Pool, DBInstance } from './static';
  `;
        /* Static files */
        const dynamicFS = {
            [path.join(outputDir, './pool.ts')]: poolLib,
            [path.join(outputDir, './seed.ts')]: seedLib,
        };
        /**
         * Write dynamic files to file system.
         */
        try {
            const compiledFS = yield vfs_1.compileVFS(dynamicFS, {
                module: typescript_1.ModuleKind.CommonJS,
                target: typescript_1.ScriptTarget.ES2016,
                lib: ['lib.esnext.d.ts', 'lib.dom.d.ts'],
                declaration: true,
                sourceMap: true,
                suppressOutputPathCheck: false,
            });
            yield vfs_1.writeToFS(compiledFS);
        }
        catch (err) /* istanbul ignore next */ {
            console.log(`Error in Test Utils generation.`);
            console.error(err);
        }
        return '';
    });
}
exports.generatePrismaTestUtils = generatePrismaTestUtils;
/* Generator specification */
generator_helper_1.generatorHandler({
    onManifest() {
        return {
            prettyName: 'Prisma Test Utils',
            defaultOutput: 'node_modules/@prisma/test-utils',
            requiresGenerators: ['prisma-client-js'],
            requiresEngines: ['queryEngine', 'migrationEngine'],
        };
    },
    onGenerate(options) {
        return __awaiter(this, void 0, void 0, function* () {
            return generatePrismaTestUtils(options);
        });
    },
});
//# sourceMappingURL=generator.js.map