"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = __importDefault(require("lodash"));
const pg_1 = __importDefault(require("pg"));
const url_1 = __importDefault(require("url"));
const lift_1 = require("../lift");
const pool_1 = require("../pool");
/**
 * Creates a dmmf specific Internal Pool instance.
 *
 * @param dmmf
 */
function getPostgreSQLPool(dmmf, cwd) {
    return class extends PostgreSQLPool {
        constructor(options) {
            super(dmmf, Object.assign({ tmpPrismaSchemaPath: lift_1.getTmpPrismaSchemaPath }, options), cwd);
        }
    };
}
exports.getPostgreSQLPool = getPostgreSQLPool;
class PostgreSQLPool extends pool_1.InternalPool {
    constructor(dmmf, options, cwd) {
        super({ max: lodash_1.default.get(options, ['pool', 'max'], Infinity) });
        this.dmmf = dmmf;
        this.projectDir = cwd;
        this.getConnection = options.connection;
        this.getTmpPrismaSchemaPath = options.tmpPrismaSchemaPath;
    }
    /**
     * Creates a DB isntance.
     */
    createDBInstance(id) {
        return __awaiter(this, void 0, void 0, function* () {
            const connection = this.getConnection(id);
            const tmpPrismaSchemaPath = this.getTmpPrismaSchemaPath(id);
            const url = readPostgreSQLUrl(connection);
            const datasources = [
                {
                    name: 'db',
                    connectorType: 'postgresql',
                    url: {
                        value: url,
                        fromEnvVar: null,
                    },
                    config: {},
                },
            ];
            /* Migrate using Lift. */
            const { datamodel } = yield lift_1.migrateLift({
                id,
                datasources,
                projectDir: this.projectDir,
                tmpPrismaSchemaPath,
                dmmf: this.dmmf,
            });
            const instance = {
                url: url,
                cwd: this.projectDir,
                datamodel: datamodel,
            };
            return instance;
        });
    }
    /**
     * Delets DB instance.
     */
    deleteDBInstance(instance) {
        return __awaiter(this, void 0, void 0, function* () {
            const connection = parsePostgreSQLUrl(instance.url);
            const client = yield getPostgreSQLClient(connection);
            try {
                yield client.query(`DROP SCHEMA IF EXISTS "${connection.schema}" CASCADE;`);
            }
            catch (err) /* istanbul ignore next */ {
                throw err;
            }
        });
    }
}
/**
 * Helper functions.
 */
/**
 * Returns a Postgres URL of the database from pool options.
 * @param options
 */
function readPostgreSQLUrl(connection) {
    return `postgres://${connection.user}:${connection.password}@${connection.host}:${connection.port}/${connection.database}?schema=${connection.schema}`;
}
/**
 * Parses a PostgreSQL url.
 * @param url
 */
function parsePostgreSQLUrl(urlStr) {
    const { query, auth, port: rawPort, hostname: host, pathname } = url_1.default.parse(urlStr, true);
    const port = parseInt(rawPort, 10);
    const [, user, password] = auth.match(/(\w+):(\w+)/);
    const [, database] = pathname.match(/\/(\w+)/);
    /* istanbul ignore next */
    if (typeof query.schema !== 'string') {
        throw new Error(`Unsupported schema type: ${typeof query.schema}`);
    }
    const schema = query.schema;
    return {
        user,
        password,
        host: host,
        port,
        database,
        schema,
    };
}
/**
 * Returns a Postgres Client from the pool configuration and makes
 * sure that the connection is established.
 *
 * @param connection
 */
function getPostgreSQLClient(connection) {
    return __awaiter(this, void 0, void 0, function* () {
        const client = new pg_1.default.Client({
            host: connection.host,
            port: connection.port,
            user: connection.user,
            password: connection.password,
            database: connection.database,
        });
        /* Establishes a connection before returning the instance. */
        try {
            yield client.connect();
            return client;
        }
        catch (err) /* istanbul ignore next */ {
            throw err;
        }
    });
}
//# sourceMappingURL=postgres.js.map