"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = __importStar(require("fs"));
const lodash_1 = __importDefault(require("lodash"));
const os = __importStar(require("os"));
const path = __importStar(require("path"));
const util_1 = require("util");
const lift_1 = require("../lift");
const pool_1 = require("../pool");
const fsUnlink = util_1.promisify(fs.unlink);
/**
 * Creates a dmmf specific Internal Pool instance.
 *
 * @param dmmf
 */
function getSQLitePool(dmmf, cwd) {
    return class extends SQLitePool {
        constructor(options) {
            super(dmmf, Object.assign({ databasePath: getTmpSQLiteDB, tmpPrismaSchemaPath: lift_1.getTmpPrismaSchemaPath }, options), cwd);
        }
    };
}
exports.getSQLitePool = getSQLitePool;
class SQLitePool extends pool_1.InternalPool {
    constructor(dmmf, options, cwd) {
        super({ max: lodash_1.default.get(options, ['pool', 'max'], Infinity) });
        this.dmmf = dmmf;
        this.projectDir = cwd;
        this.getDatabasePath = options.databasePath;
        this.getTmpPrismaSchemaPath = options.tmpPrismaSchemaPath;
    }
    /**
     * Creates a new DB instances and lifts a migration.
     */
    createDBInstance(id) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                /* Constants */
                const dbFile = this.getDatabasePath(id);
                const tmpPrismaSchemaPath = this.getTmpPrismaSchemaPath(id);
                const datasources = [
                    {
                        name: 'db',
                        connectorType: 'sqlite',
                        url: {
                            value: `file:${dbFile}`,
                            fromEnvVar: null,
                        },
                        config: {},
                    },
                ];
                /* Migrate using Lift. */
                const { datamodel } = yield lift_1.migrateLift({
                    id,
                    projectDir: this.projectDir,
                    datasources,
                    tmpPrismaSchemaPath,
                    dmmf: this.dmmf,
                });
                const instance = {
                    url: dbFile,
                    cwd: this.projectDir,
                    datamodel: datamodel,
                };
                return instance;
            }
            catch (err) /* istanbul ignore next */ {
                throw err;
            }
        });
    }
    /**
     * Deletes the db files.
     *
     * @param instance
     */
    deleteDBInstance(instance) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield fsUnlink(instance.url);
            }
            catch (err) /* istanbul ignore next */ {
                throw err;
            }
        });
    }
}
/**
 * Allocates a new space in the tmp dir for the db instance.
 *
 * @param id
 */
function getTmpSQLiteDB(id) {
    const tmpDir = os.tmpdir();
    const dbFile = path.join(tmpDir, `./prisma-sqlite-${id}-db.db`);
    return dbFile;
}
exports.getTmpSQLiteDB = getTmpSQLiteDB;
//# sourceMappingURL=sqlite.js.map