import { Pool, DBInstance } from '../types';
export declare type PoolOptions = {
    max: number;
};
export declare abstract class InternalPool implements Pool {
    protected dbs: {
        booting: string[];
        busy: DBInstance[];
    };
    protected waiters: Waiter[];
    protected capacity: number;
    constructor(options: PoolOptions);
    /**
     * A required method used to create new databases in the pool.
     */
    protected abstract createDBInstance(id: string): Promise<DBInstance>;
    /**
     * A required methdo used to delete databases in the pool.
     *
     * @param instance
     */
    protected abstract deleteDBInstance(instance: DBInstance): Promise<void>;
    /**
     * Creates a new DBInstance and occupies the space in the pool.
     */
    getDBInstance(): Promise<DBInstance>;
    /**
     * Releases a db isntance in the pool and triggers the creation
     * of new instance if there are waiters.
     *
     * @param db
     */
    releaseDBInstance(instance: DBInstance): Promise<void>;
    /**
     * A function wrapper which makes sure that the db instance is
     * correctly allocated prior to execution of the function, and correctly
     * released after the execution has finished.
     *
     * @param fn
     */
    run<T>(fn: (db: DBInstance) => Promise<T>): Promise<T>;
    /**
     * Rejects remaining waiters, and releases all
     * remaining instances in the pool.
     */
    drain(): Promise<void>;
}
declare class Waiter {
    private promise;
    private resolve?;
    private reject?;
    constructor();
    wait(): Promise<DBInstance>;
    allocate(instance: DBInstance): void;
    fail(reason?: any): void;
}
export {};
