"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Fallbacks to default if undefined.
 *
 * @param fallback
 * @param val
 */
function withDefault(fallback, val) {
    if (val !== undefined && val !== null)
        return val;
    return fallback;
}
exports.withDefault = withDefault;
/**
 * Maps values and filters nulls.
 *
 * @param f
 * @param xs
 */
function filterMap(xs, f) {
    return xs.map(f).filter(notNull);
}
exports.filterMap = filterMap;
/**
 * Determines whether a values is null.
 * @param val
 */
function notNull(val) {
    return val !== null;
}
exports.notNull = notNull;
/**
 * Negates the boolean function.
 * @param fn
 */
function not(fn) {
    return (...params) => !fn(...params);
}
exports.not = not;
/**
 * Filters keys from an object.
 * @param dict
 * @param fn
 */
function filterKeys(dict, fn) {
    return Object.keys(dict)
        .filter(key => fn(key, dict[key]))
        .reduce((acc, key) => (Object.assign(Object.assign({}, acc), { [key]: dict[key] })), {});
}
exports.filterKeys = filterKeys;
/**
 * Filters keys from an object.
 * @param dict
 * @param fn
 */
function mapEntries(dict, fn) {
    return Object.keys(dict).reduce((acc, key) => (Object.assign(Object.assign({}, acc), { [key]: fn(dict[key], key) })), {});
}
exports.mapEntries = mapEntries;
//# sourceMappingURL=utils.js.map