"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const typescript_1 = require("typescript");
const util_1 = require("util");
const mkdir = util_1.promisify(fs.mkdir);
const writeFile = util_1.promisify(fs.writeFile);
const copyFile = util_1.promisify(fs.copyFile);
const readDir = util_1.promisify(fs.readdir);
/**
 * Searches virtual file system for TS files and compiles them.
 * Leaves non-TS files untacked.
 *
 * @param vfs
 */
function compileVFS(vfs, options) {
    return __awaiter(this, void 0, void 0, function* () {
        /* Files */
        const tsFiles = Object.keys(vfs).filter(file => file.endsWith('.ts'));
        const nonTsFiles = Object.keys(vfs).filter(file => !file.endsWith('.ts'));
        const compiledVFS = nonTsFiles.reduce((acc, file) => (Object.assign(Object.assign({}, acc), { [file]: vfs[file] })), {});
        /* Compiler Configuration */
        const compilerHost = typescript_1.createCompilerHost(options);
        const { getSourceFile, readFile } = compilerHost;
        compilerHost.getSourceFile = (fileName, target) => {
            if (vfs.hasOwnProperty(fileName)) {
                return typescript_1.createSourceFile(fileName, vfs[fileName], target, true);
            }
            else {
                return getSourceFile(fileName, target);
            }
        };
        compilerHost.writeFile = (fileName, data) => {
            compiledVFS[fileName] = data;
        };
        /* Run the compiler */
        const program = typescript_1.createProgram(tsFiles, options, compilerHost);
        program.emit();
        return compiledVFS;
    });
}
exports.compileVFS = compileVFS;
/**
 * Copies files form vfs to specified locations.
 *
 * @param vfs
 */
function copyVFS(vfs) {
    return __awaiter(this, void 0, void 0, function* () {
        /* Find folders. */
        const folders = Object.keys(vfs).filter(file => fs.existsSync(vfs[file]) && fs.lstatSync(vfs[file]).isDirectory());
        if (folders.length > 0) {
            /* Resolve folders. */
            const resolvedVFS = yield Object.keys(vfs).reduce((acc, outPath) => acc.then((accVfs) => __awaiter(this, void 0, void 0, function* () {
                const inPath = vfs[outPath];
                if (fs.existsSync(inPath) && fs.lstatSync(inPath).isDirectory()) {
                    /* Replace dir with resolved path. */
                    const files = yield readDir(inPath);
                    return files.reduce((acc, file) => (Object.assign(Object.assign({}, acc), { [path.join(outPath, file)]: path.join(inPath, file) })), accVfs);
                }
                else {
                    /* Insert files. */
                    return Object.assign(Object.assign({}, accVfs), { [outPath]: inPath });
                }
            })), Promise.resolve({}));
            return copyVFS(resolvedVFS);
        }
        else {
            /* Copy files. */
            const actions = Object.keys(vfs).map((outPath) => __awaiter(this, void 0, void 0, function* () {
                yield mkdir(path.dirname(outPath), {
                    recursive: true,
                });
                yield copyFile(vfs[outPath], outPath);
            }));
            try {
                yield Promise.all(actions);
            }
            catch (err) /* istanbul ignore next */ {
                throw err;
            }
        }
    });
}
exports.copyVFS = copyVFS;
/**
 * Writes virtual file system representation to the file system.
 *
 * @param vfs
 */
function writeToFS(vfs) {
    return __awaiter(this, void 0, void 0, function* () {
        const actions = Object.keys(vfs).map((filePath) => __awaiter(this, void 0, void 0, function* () {
            yield mkdir(path.dirname(filePath), { recursive: true });
            yield writeFile(filePath, vfs[filePath]);
        }));
        try {
            yield Promise.all(actions);
        }
        catch (err) /* istanbul ignore next */ {
            throw err;
        }
    });
}
exports.writeToFS = writeToFS;
//# sourceMappingURL=vfs.js.map