/// <reference types="chance" />
import { Dictionary } from 'lodash';
/**
 * Types used to describe the seed models.
 */
export declare type SeedFunction<GeneratedPrismaClientType extends PrismaClientType, GeneratedSeedModels extends SeedModels> = (options: SeedOptions<GeneratedPrismaClientType, GeneratedSeedModels>) => Promise<Dictionary<object[]>>;
export declare type SeedModelsDefinition<GeneratedSeedModels extends SeedModels> = (kit: SeedKit) => GeneratedSeedModels;
export interface SeedKit {
    faker: Chance.Chance;
}
export declare type SeedModels = {
    '*': {
        amount: number;
    };
    [type: string]: SeedModel;
};
export declare type SeedModel = {
    amount?: number;
    factory?: Dictionary<SeedModelFieldDefinition | (() => SeedModelFieldDefinition) | SeedModelScalarListDefinition | SeedModelFieldRelationConstraint>;
};
export declare type SeedModelFieldDefinition = string | number | boolean;
export declare type SeedModelScalarListDefinition = SeedModelFieldDefinition[] | (() => SeedModelFieldDefinition[]);
export declare type SeedModelFieldRelationConstraint = {
    min?: number;
    max?: number;
};
/**
 * Represents the outline of the Prisma Client functions.
 */
export declare type PrismaClientType = any;
export declare type SeedOptions<GeneratedPrismaClientType extends PrismaClientType, GeneratedSeedModel extends SeedModels> = {
    client: GeneratedPrismaClientType;
    models: SeedModelsDefinition<GeneratedSeedModel>;
    seed?: number;
};
/**
 * Types used to describe the Pool.
 */
export declare type DBInstance = {
    url: string;
    cwd: string;
    datamodel: string;
};
export declare abstract class Pool {
    abstract getDBInstance(): Promise<DBInstance>;
    abstract releaseDBInstance(db: DBInstance): Promise<void>;
    abstract run<T>(fn: (db: DBInstance) => Promise<T>): Promise<T>;
    abstract drain(): Promise<void>;
}
